package code;

import heap.*;

public interface Expression {
	
	/** This expression cannot be evaluated because of integer variables or non validable pointer expression in it */
	public static final int double_ = 0;
	
	/** This expression is evaluated to true */
	public static final int true_ 	 = 1;
	
	/** This expression is evaluated to false */
	public static final int false_  = 2;
	
	/**
	 * Evaluate this expression for the state of memory given. This can only be done if the expression is pure.
	 * 
	 * @param	h	The state of memory needed for evaluation
	 * @return	if expression is pure true_/false_ else double_
	 * @see code.Expression#double_
	 * @see code.Expression#true_
	 * @see code.Expression#false_
	 * @see code.Expression#isPure()
	 * */
	public int evaluate(Heap h);
	
	/**
	 * Check if this expression contains only memory predicates
	 * 
	 *  @return True if there's only memory predicates.
	 * */
	public boolean isPure();	
	
}
