package heap;

/**Represents a vertex of the Symbolic Shape Graph. It is defined by a counter and an Id.
 * 
 *  @author Swann.Perarnau@imag.fr
 *  */
public class Vertex {
    private String counterName;
    private int fileid;
    private int id;
    private static int nextid = 0;
    
    public Vertex(String cn, int f) {
        this.counterName = cn;
        this.fileid = f;
        this.id = nextid;
        nextid++; 
    }
    
    public Vertex(String cn)
    {
    	this.counterName = cn;
    	this.fileid = 900+nextid;
    	this.id = nextid;
        nextid++; 
    }
    
    public Vertex(Vertex v)
    {
    	this.counterName = new String(v.counterName);
    	this.fileid = v.fileid;
    	this.id = v.id;
    }
    
	public String counter() {
        return counterName;
    }
   
    public int fileid() {
        return fileid;
    }
    
    public int getId(){
    	return id;
    }
    
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final Vertex other = (Vertex) obj;
        if (counterName == null) {
            if (other.counterName != null)
                return false;
        } else if (!counterName.equals(other.counterName))
            return false;
        return true;
    }
    
    
    
    
    
    public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((counterName == null) ? 0 : counterName.hashCode());
		result = PRIME * result + fileid;
		result = PRIME * result + id;		
		return result;
	}

	

	public String toString()
    {
    	return new String(fileid + " : " + counterName + "\n");
    }
    
    
    
}
