package main;
import java.io.*;
import java.util.*;

import automaton.*;
import heap.*;
import heap.parser.*;
import code.*;
import code.parser.*;
import output.*;
//import qsl.parser.*;
//import qsl.*;

public class OptionHelper {

		private Properties configuration;
	
	
	
		private String heap_filename;
		private String program_filename;
//		private String qsl_filename;
		
		private String out_filename;
		private String out_format;
		
		//private boolean logging;
		
		private HeapParser hp;
		private CodeParser cp;
		private Automaton automaton;
		private Display display;
		
		public OptionHelper() {
			configuration = new Properties();
			
			try {
				configuration.load(new FileInputStream("options.properties"));
			} catch (FileNotFoundException e) {
				System.err.println("Options Properties file not found !");
				e.printStackTrace();
				System.exit(1);
			} catch (IOException e) {
				System.err.println("Cannot read options properties file !");
				e.printStackTrace();
				System.exit(1);
			}
		}
		
		public void LoadOptions(String args[]) {
//			if(configuration.getProperty("mode").equalsIgnoreCase("debug"))
//			{
//				// doing something else than computing an automaton
//				qsl_filename = args[0];
//				qsl_test(qsl_filename);				
//				System.exit(0);
//			}
			
			
			if(args.length != 4)
			{
				System.out.println("Usage : $l2ca heap program format output_filname");
				System.exit(1);
			}
				
			heap_filename = args[0];
			program_filename = args[1];
			out_format = args[2];
			out_filename = args[3];
			
			try {
				display = new Display(out_format,out_filename);
			} catch (Exception e) {
				e.printStackTrace();
				System.exit(1);
			}
		}
		
		
		public boolean pointerGuards()
		{
			return display.pointerGuards();
		}
		
		
		public void setHeap_filename(String heap_filename) {
			this.heap_filename = heap_filename;
		}

		public void setOut_filename(String out_filename) {
			this.out_filename = out_filename;
		}

		public void setOut_format(String out_format) {
			this.out_format = out_format;
		}

		public void setProgram_filename(String program_filename) {
			this.program_filename = program_filename;
		}

//		public void setQsl_filename(String qsl_filename) {
//			this.qsl_filename = qsl_filename;
//		}

		public Heap ParseHeap() {
			try {
				hp = new HeapParser(heap_filename);
			}
			catch(java.io.FileNotFoundException e) {
				System.out.println("Heap Parser Version "+ HeapParser.version +" : file "+ heap_filename +" not found.");
				System.exit(1);
			}
			hp.parse();
			hp.computeMap();
			hp.getHeap().normalize();
			return hp.getHeap();
		}
		
		public CodeNode ParseProgram() {
			try {
				cp = new CodeParser(program_filename);
			}
			catch(java.io.FileNotFoundException e) {
				System.out.println("Code Parser Version "+ CodeParser.version +" : file "+ program_filename +" not found.");
				System.exit(1);
			}
			cp.parse();
			cp.typeCheck();
			cp.setNext();
			return cp.getProgram();
		}
		
		public void Link() {
			Map m = cp.types();
			Set s = m.entrySet();
			Iterator i = s.iterator();
			while(i.hasNext())
			{
				Map.Entry tmp = (Map.Entry)i.next();
				if(((Integer)tmp.getValue()).intValue() == code.CodeNode.POINTER)
					hp.getHeap().add(new Root((String)tmp.getKey()));
			}
		}
		
		private List IntList(){
			List l = new LinkedList();
			Map m = cp.types();
			Set s = m.entrySet();
			Iterator i = s.iterator();
			while(i.hasNext())
			{
				Map.Entry tmp = (Map.Entry)i.next();
				if(((Integer)tmp.getValue()).intValue() == code.CodeNode.INTEGER)
					l.add((String)tmp.getKey());
			}
			return l;
		}

		public Automaton ComputeAutomaton() {
			System.out.println("Computing automaton...");
			automaton = new Automaton();
			automaton.generate(hp.getHeap(), cp.getProgram(),IntList());
			System.out.println("Automaton computed successfully.");
			
			//System.out.println("Minimizing automaton...");
			//automaton.minimize();
			//System.out.println("Automaton minimized successfully.");
			return automaton;				
		}
		
		public void Display() {
			System.out.println("Printing automaton on file : "+display.filename());
			display.write(automaton);
			System.out.println("Automaton printed successfully.");
		}
		
		public void Debug(Automaton a) {
			Display d = display;
			try {
				d = new Display(out_format,"debug/"+System.currentTimeMillis());
			} catch (Exception e) {
				e.printStackTrace();
				System.exit(1);
			}
			System.out.println("Printing automaton on file : "+d.filename());
			d.write(automaton);
			System.out.println("Automaton printed successfully.");
		}
		
		/*private void qsl_test(String filename)
		{
			QSLParser p = null;
			try {
				p = new QSLParser(filename);
			}
			catch(java.io.FileNotFoundException e) {
				System.out.println("QSL Parser Version "+ QSLParser.version +" : file "+ filename +" not found.");
				System.exit(1);
			}
			p.parse();
			QSLNode tree = p.getTree();
			System.out.println("Original Tree");
			tree.dump(">");
			tree.NNF();
			System.out.println("NNF Tree");
			tree.dump(">");
			tree.idPrunning(new HashMap());
			System.out.println("Prunning Tree");
			tree.dump(">");
			tree.QNF();
			System.out.println("QNF Tree");
			tree.dump(">");
			tree.DNF();
			System.out.println("DNF Tree");
			tree.dump(">");
			tree.starChange();
			System.out.println("StarChange Tree");
			tree.dump(">");
		}*/
		
		
}
