package output;
import automaton.*;

public class Display {
	public static final String DOT = "dot";
	public static final String FAST = "fast";
	public static final String ARMC = "armc";
        public static final String CA = "ca";
	
	private FileOutput module;
	
	public Display(String mode,String name) throws Exception
	{
		if(mode.equalsIgnoreCase(DOT))
		{
			System.out.println("Loading DOT output module...");
			module = new DotFile();
			module.open(name);
			System.out.println("DOT output module loaded.");
		}
		else if(mode.equalsIgnoreCase(FAST))
		{
			System.out.println("Loading FAST output module...");
			module = new FastFile();
			module.open(name);
			System.out.println("FAST output module loaded.");
		}
		else if(mode.equalsIgnoreCase(ARMC))
		{
			System.out.println("Loading ARMC output module...");
			module = new ArmcFile();
			module.open(name);
			System.out.println("ARMC output module loaded.");
		}
		else if (mode.equalsIgnoreCase(CA)) 
		{
			System.out.println("Loading CA output module...");
			module = new CaFile();
			module.open(name);
			System.out.println("CA output module loaded.");
		}
		else
			throw new Exception("No such output mode : " + mode);
	}
	
	public boolean pointerGuards()
	{
		return module.pointerGuards();
	}
	
	public String filename(){
		return module.filename();
	}
	
	public void write(Automaton a)
	{
		module.write(a);
		module.close();
	}
	
}
