package output;

import java.util.*;
import code.*;
import automaton.*;

import automaton.Automaton;

public class FastFile implements FileOutput {
	private java.io.PrintWriter file;
	private String name;
	
	public FastFile()
	{
		file = null;
		name = null;
	}
	
	public String add() {
		return " + ";
	}

	public String and() {
		return " && ";
	}

	public String asgn() {
		return "' = ";
	}

	public void close() {
		file.flush();
		file.close();
	}

	public String eq() {
		return " = ";
	}

	public String ge() {
		return " >= ";
	}

	public String gt() {
		return " > ";
	}

	public String le() {
		return " <= ";
	}

	public String lt() {
		return " < ";
	}

	public String mult() {
		return " * ";
	}

	public String not() {
		return " ! ";
	}

	public void open(String name) throws java.io.FileNotFoundException {
		this.name = name;
		file = new java.io.PrintWriter(new java.io.FileOutputStream(name+".fast"));
	}
	
	public String filename(){
		return name+".fast";
	}

	public String or() {
		return " || ";
	}

	public String slist() {
		return " , ";
	}

	public String sub() {
		return " - ";
	}
	
	public String Null(){
		return " null ";
	}
	
	public String New(){
		return " new ";
	}

	public void write(Automaton a) {
		file.println("model "+name+" { ");
		
		// Print all vars of this automaton
		this.file.print("\t var " + this.stubCounter());
		Iterator i = a.getCounters().iterator();
		while(i.hasNext())
		{
			this.file.print(", "+(String)i.next());
		}
		this.file.println(";");
		
		// Print all states
		this.file.print("\t states h_error");
		i = a.getStates().iterator();
		while(i.hasNext())
		{
			State s = (State)i.next();
			if (!s.isError())
				this.file.print(", h_" + s.getId());			
		}
//		i = a.getTransitionStates().iterator();
//		while(i.hasNext())
//		{
//			State s = (State)i.next();
//			this.file.print(", h_" + s.getId());
//		}	
		this.file.println(";");
		
		i = a.getTransitions().iterator();
		int j = 0;
		while(i.hasNext())
		{
			this.file.println("transition t"+j+" := {");
			this.writeTransition((Transition)i.next());
			this.file.println("};");
			j++;
		}
		this.file.println("}");
		this.file.println(this.writeStrategy(a));
	}
	

	private String stubCounter(){
		return "f__v";
	}
	
	private void writeTransition(Transition t)
	{
		if(t.getSinitial().isError())
			this.file.println("from := h_error;");
		else
			this.file.println("from := h_"+t.getSinitial().getId()+";");
		
		if(t.getSfinal().isError())
			this.file.println("to := h_error;");
		else
			this.file.println("to := h_"+t.getSfinal().getId()+";");
		
		this.file.println("guard := "+this.writeGuard(t.getGuard())+";");
		this.file.println("action := "+this.writeActions(t.getActions())+";");
	}
	
	private String writeGuard(CodeNode g)
	{
		if(g == null)
			return "true";
		else
			return g.out(this);
	}
	
	private String writeActions(CodeNode a)
	{
		if(a == null)
			return this.stubCounter()+"' = "+this.stubCounter();
		else
			return a.out(this);
	}
	
	private String writeStrategy(Automaton a)
	{
		String s = "";
		s += "strategy base {\n";
		s += "Region init := {state = h_"+a.getFirst().getId()+"};\n";
		s += "Region bad := {state = h_error};\n";
		s += "Transitions t := {";
		
		int i=0;
		for(i=0;i<a.getTransitions().size()-1;i++)
			s +="t"+i+",";
		s += "t"+i+"};\n";
		
		s += "Region reach := post*(init, t, 2);\n";
		s += "if ( isEmpty(reach && bad) )\n\tthen print(\"OK\");\n\telse print(\"KO\");\nendif\n";

		s += "}";
		return s;
	}
	
	public boolean pointerGuards()
	{
		return false;
	}
	
	public void switchNot(){
	}

	public String leftP() {
		return " ( ";
	}

	public String rightP() {
		return " ) ";
	}

	public boolean supportSwitch() {
		return false;
	}
}
